@@ -17,6 +17,8 @@ public:
 	void setBgColor(QColor color);
 	void setP1Color(QColor color);
 	void setP2Color(QColor color);
+        void setTextColor(QColor color);
+        void setText(const QString& txt);
 
 protected:
 	void paintEvent(QPaintEvent *event);
@@ -26,10 +28,12 @@ private:
 	QColor m_bgcolor;
 	QColor m_p1color;
 	QColor m_p2color;
+        QColor m_textColor;
 	int m_width;
 	int m_height;
 	int m_pos1;
 	int m_pos2;
+        QString m_strText;
 
 signals:
     void positionChanged(int);
