@@ -6,6 +6,8 @@
 #include <QtGui/QWidget>
 #include <QMainWindow>
 #include <QtGui/QVBoxLayout>
+#include <QAction>
+#include <QMenuBar>
 #include <QTableWidget>
 #include <QTableWidgetItem>
 #include <QStringList>
@@ -61,6 +63,8 @@ private:
 
     QueueView *m_pQueueView;
 
+    QAction *m_pActSortByPopularity;
+
     QTableWidget m_TrackInfo;
 
     qint32 m_currSortCol; //current sort column
@@ -77,6 +81,7 @@ private slots:
     void OnTrackActivated(QTableWidgetItem* trackItem);
     void OnPlayQueuePosUpdated(qint32 pos);
     void OnHeaderClicked(int logicalIndex);
+    void OnSortByPopularityTriggered(bool checked);
 
 signals:
 
