@@ -17,14 +17,6 @@
  */
 Spotify::Session* Spotify::Session::callingSession;
 
-//externals (shared globals)
-extern QList<QByteArray> g_FIFO; //FIFO datastructure containing audio data to be played
-extern qint32 g_FIFOSize; //current size of audio FIFO (in bytes)
-extern QMutex g_audio_mutex; //mutex for buffer synchronisation between main and playback threads
-extern bool g_isPaused; //true in case music buffering is currently paused (set by main session thread)
-extern bool g_playbackPaused; //true in case e.g., user has requested to pause playback of current track
-extern bool g_endOfTrack; //true when current end of track has been reached
-
 Spotify::Session::Session( QObject* parent ):
     QObject( parent ),
     m_currentTrack( 0 ),
@@ -60,8 +52,8 @@ Spotify::Session::Session( QObject* parent ):
             this, SLOT(OnImageLoaded(sp_image*, void*)),Qt::QueuedConnection);
     connect(this, SIGNAL(metadataUpdated()),
                     this, SLOT(updatePlaylistsMetadataSlot()),Qt::QueuedConnection);
-    connect(this, SIGNAL(onMusicDelivery(const sp_audioformat*,QByteArray)),
-                    this, SLOT(OnMusicDelivery(const sp_audioformat*,QByteArray)),Qt::QueuedConnection);
+    connect(this, SIGNAL(onMusicDelivery(int, const sp_audioformat*,QByteArray)),
+                    this, SLOT(OnMusicDelivery(int, const sp_audioformat*,QByteArray)),Qt::QueuedConnection);
     connect(this, SIGNAL(onSearchComplete(sp_search*)),
                     this, SLOT(OnSearchComplete(sp_search*)),Qt::QueuedConnection);
 
@@ -69,6 +61,8 @@ Spotify::Session::Session( QObject* parent ):
 
     m_pAudioThread = NULL;
 
+    m_preBufActive = true;
+
     connect(m_pProcEvtTimer,SIGNAL(timeout()),this,SLOT(OnProcessThreadEvents()),Qt::QueuedConnection);
 
 }
@@ -95,11 +89,11 @@ void Spotify::Session::init()
     m_ProcessEvents = false; //event processing not initiated
     m_DisableNotifyThread = false;
 
-   g_isPaused = false;
-   g_playbackPaused = false;
-
    m_pAudioThread = new PlaybackThread(this);
 
+   m_pAudioThread->setBufferingPaused(false);
+   m_pAudioThread->setPlaybackPaused(false);
+
     connect(m_pAudioThread, SIGNAL(resumeBuffering()),
                     this, SLOT(OnResumeBuffering()),Qt::QueuedConnection);
     connect(m_pAudioThread, SIGNAL(playbackPositionUpdated(qint64)),
@@ -112,6 +106,88 @@ void Spotify::Session::init()
    m_pAudioThread->start(); //start audio playback thread
 }
 
+void Spotify::Session::setMinBufferSize(qint32 secs)
+{
+    //set the (minimum) size of the internal FIFO used to hold audio data
+    if(m_pAudioThread)
+    {
+        m_pAudioThread->getMutex()->lock();
+        m_pAudioThread->setMinBufferSize(secs);
+        m_pAudioThread->getMutex()->unlock();
+    }
+}
+
+void Spotify::Session::setPreBufferingActive(bool active)
+{
+    //set whether track pre-buffering is active (i.e., buffer next track data in advance)
+
+    if(m_pAudioThread)
+    {
+        m_pAudioThread->getMutex()->lock();
+        qDebug() << "Set pre-buffering state: " << active;
+        m_preBufActive = active;
+        m_pAudioThread->setPreBufActive(active);
+        //m_pAudioThread->setNextTrackStartPos(-1);
+        m_pAudioThread->getMutex()->unlock();
+    }
+}
+
+bool Spotify::Session::getPreBufferingActive()
+{
+    return m_preBufActive;
+}
+
+void Spotify::Session::setPlaybackPos(qint64 pos)
+{
+    m_trackPos = pos;
+}
+
+void Spotify::Session::setBufferingPos(qint64 pos)
+{
+    m_bufferPos = pos;
+}
+
+qint64 Spotify::Session::getBufferingPos()
+{
+    //returns the current track buffering position
+    if(!m_bufferPosList.isEmpty())
+    {
+        return m_bufferPosList.first();
+    }
+    else
+    {
+        return m_bufferPos;
+    }
+}
+
+void Spotify::Session::setPlaying(bool playing)
+{
+    m_playing = playing;
+}
+
+void Spotify::Session::setNextTrackStartPos()
+{
+    //set the next track start position in the audio FIFO
+    if(m_pAudioThread)
+    {
+        m_pAudioThread->getMutex()->lock();
+        m_pAudioThread->setNextTrackStartPos();
+        m_pAudioThread->getMutex()->unlock();
+    }
+}
+
+void Spotify::Session::setNewTrackStarted(bool started)
+{
+    //indicate start of new track audio buffer data (in playback thread FIFO)
+    if(m_pAudioThread)
+    {
+        m_pAudioThread->getMutex()->lock();
+        m_pAudioThread->setNewTrackStarted(started);
+        m_pAudioThread->getMutex()->unlock();
+    }
+}
+
+
 void Spotify::Session::OnPlayingState(bool state)
 {
     //emit current playback state (playing / paused)
@@ -191,7 +267,7 @@ void Spotify::Session::OnPlaybackPositionUpdated(qint64 duration)
     emit playbackPositionUpdated(m_trackPos);
 }
 
-Spotify::Error Spotify::Session::play( Spotify::Track* track )
+Spotify::Error Spotify::Session::play( Spotify::Track* track, bool preBuffer )
 {
     m_playing = false;
     m_currentTrack = track;
@@ -199,13 +275,6 @@ Spotify::Error Spotify::Session::play( Spotify::Track* track )
     QMutex mutex;
     QWaitCondition sleep;
 
-    //sleep.wait(&mutex,850);
-
-    // Clean up
-   // qDebug() << "+unload";
-   // sp_session_player_unload( m_session );
-   // qDebug() << "-unload";
-
     sleep.wait(&mutex,1300);
 	
     // Disregard tracks that are not available:
@@ -216,7 +285,7 @@ Spotify::Error Spotify::Session::play( Spotify::Track* track )
     }
 
 
-    // Disregard unloaded og unavailable tracks:
+    // Disregard unloaded of unavailable tracks:
     if( !track->isLoaded() )
     {
         qDebug() << "ERROR: Asked to play back an unloaded track..";
@@ -231,13 +300,20 @@ Spotify::Error Spotify::Session::play( Spotify::Track* track )
         return error;
     }
 
-    g_audio_mutex.lock();
-    g_FIFO.clear();
-    g_FIFOSize = 0;
-    g_isPaused = false;
-    g_playbackPaused = false;
-    g_endOfTrack = false;
-    g_audio_mutex.unlock();
+    if(!preBuffer) //pre-buffering not active (i.e., audio FIFO should be cleared)
+    {
+        if(m_pAudioThread)
+        {
+            m_pAudioThread->getMutex()->lock();
+            m_pAudioThread->clearFIFO();
+            m_pAudioThread->setFIFOSize(0);
+            m_pAudioThread->setBufferingPaused(false);
+            m_pAudioThread->setPlaybackPaused(false);
+            m_pAudioThread->setEndOfTrack(false);
+            //m_pAudioThread->setNextTrackStartPos(-1);
+            m_pAudioThread->getMutex()->unlock();
+        }
+    }
 
     //sleep.wait(&mutex,300);
 
@@ -245,10 +321,13 @@ Spotify::Error Spotify::Session::play( Spotify::Track* track )
     qDebug() << "+play";
     error = sp_session_player_play( m_session, true );
     qDebug() << "-play";
-    if( !error.isError() )
+    if( !error.isError())
     {
-        m_trackPos = 0;
-        m_bufferPos = 0;
+        if(!preBuffer) //track pre-buffer not active
+        {
+            m_trackPos = 0;
+            m_bufferPos = 0;
+        }
         m_playing = true;
     }
     else
@@ -288,12 +367,12 @@ Spotify::Error Spotify::Session::stop()
 void Spotify::Session::pausePlayback()
 {
     //Pause / resume playback from Spotify (e.g., signal to playback thread to stop / continue writing data to PulseAudio)
-    g_audio_mutex.lock();
-    if(g_playbackPaused)
-        g_playbackPaused = false;
+    m_pAudioThread->getMutex()->lock();
+    if(m_pAudioThread->getPlaybackPaused())
+       m_pAudioThread->setPlaybackPaused(false);
     else
-        g_playbackPaused = true;
-    g_audio_mutex.unlock();
+        m_pAudioThread->setPlaybackPaused(true);
+    m_pAudioThread->getMutex()->unlock();
 }
 
 Spotify::Error Spotify::Session::pause()
@@ -1144,43 +1223,53 @@ int Spotify::Session::musicDeliveryWrapper( const sp_audioformat* format, const
 
     QByteArray buf((const char*)frames,bufSize);
 
-    emit onMusicDelivery(format,buf);
-
-    m_bufferPos += (num_frames * 1000.0)/format->sample_rate;
-    emit trackPositionUpdated( m_bufferPos );
+    emit onMusicDelivery(num_frames,format,buf);
 
     return num_frames;
 }
 
-void Spotify::Session::OnMusicDelivery(const sp_audioformat* format, QByteArray buffer)
+void Spotify::Session::OnMusicDelivery(int num_frames, const sp_audioformat* format, QByteArray buffer)
 {
-    //Audio data bufferered received, add buffered data at end of audio playback queue
+    //Audio data buffer received, add buffered data at end of audio playback queue
 
     QByteArray bufPart;
 
-    g_audio_mutex.lock();
+    m_pAudioThread->getMutex()->lock();
+
+    if(m_preBufActive)
+    {
+        if(m_pAudioThread->getNewTrackStarted())
+        {
+            qDebug() << "Pre-buffering active; next track started...";
+            m_pAudioThread->setNewTrackStarted(false); //reset track started state
+            m_pAudioThread->setNextTrackStartPos();
+            m_bufferPosList.append(m_bufferPos);
+            m_bufferPos = 0; //reset buffering postion
+        }
+    }
+
+    m_bufferPos += (num_frames * 1000.0)/format->sample_rate;
+    emit trackPositionUpdated( m_bufferPos );
 
     //split audio data buffer into max 1 sec chunks
 
     for(int bufPos = 0; bufPos < buffer.size(); bufPos+=format->sample_rate*sizeof(int16_t)*format->channels)
     {
         bufPart = buffer.mid(bufPos,format->sample_rate*sizeof(int16_t)*format->channels);
-        g_FIFO.append(bufPart);
+        m_pAudioThread->FIFOAppend(bufPart);
     }
 
-    g_FIFOSize+=buffer.size();
+    m_pAudioThread->incFIFOSize(buffer.size());
 
-    //qDebug()<<"Current FIFO size: " << g_FIFOSize;
-
-    if(g_FIFOSize>=35*format->sample_rate*sizeof(int16_t)*format->channels) //>=35 secs of data in buffer
+    if(m_pAudioThread->getFIFOSize()>=(m_pAudioThread->getMinBufferSize()+5)*format->sample_rate*sizeof(int16_t)*format->channels) //>=35 secs of data in buffer
     {
-        g_isPaused = true;
-        g_audio_mutex.unlock();
+        m_pAudioThread->setBufferingPaused(true);
+        m_pAudioThread->getMutex()->unlock();
         pause();
     }
     else
     {
-        g_audio_mutex.unlock();
+        m_pAudioThread->getMutex()->unlock();
     }
 }
 
@@ -1222,8 +1311,18 @@ void Spotify::Session::endOfTrackCallback( sp_session* session )
 void Spotify::Session::endOfTrackWrapper()
 {
     qDebug()<<"End of track callback..";
-    g_audio_mutex.lock();
-    g_endOfTrack = true;
-    g_audio_mutex.unlock();
+    m_pAudioThread->getMutex()->lock();
+    if(!m_preBufActive)
+    {
+        m_pAudioThread->setEndOfTrack(true);
+    }
+    else
+    {
+        if(!m_bufferPosList.isEmpty())
+            m_bufferPosList.removeFirst();
+        emit bufferNextTrack(); //pre-buffering active; signal that buffering of following track can be initiated
+    }
+
+    m_pAudioThread->getMutex()->unlock();
 }
 
