@@ -279,3 +279,15 @@ bool tracksByAlbumNameDesc(Spotify::Track* track1, Spotify::Track* track2)
     return false;
 }
 
+bool tracksByPopularityDesc(Spotify::Track* track1, Spotify::Track* track2)
+{
+    //compare track instances by popularity (descending)
+
+    if(track1 && track2)
+    {
+        return(track1->getPopularity()>track2->getPopularity());
+    }
+
+    return false;
+}
+
