@@ -158,6 +158,11 @@ namespace Spotify
             qSort(m_tracks.begin(),m_tracks.end(),tracksByAlbumNameDesc);
     }
 
+    void TrackCollection::sortByPopularity()
+    {
+        qSort(m_tracks.begin(),m_tracks.end(),tracksByPopularityDesc);
+    }
+
     void TrackCollection::restoreOrignalCollection()
     {
         //restore original track collection (after re-sort etc.)
